classdef Smtl
    % Smtl - Simulated Microtubule list
    %
    % This is the main class obj for simulating multiple MT.
    %
    % Stores information on simulation parameters and analyzed result from
    % the simulation. All simulations within this list have identical
    % simulation paramenters
    %
    %
    % THK 7/2016 Original development
    % LAM 4/2022 Modifications, updates and comments
    %
    % Abbreviations
    % btwn = between
    % MT = microtubule
    % PF = protofilament
    % T = true
    % F = false
    %
    % Matlab 2019a; Red Hat Enterprise Linux Server 7.6 Maipo
    %
    %
    
    %% Class variables / constants
    
    properties (Constant)
        % Location of fortran executable (simulation code)
        modelLoc = struct('unix', '/path/to/matlab/codes')
    end
    
    properties
        % path / location where results will be stored
        % the simulation output = filePath/simulation(#).out
        % where (#) is an integer.
        filePath
        
        % name of the file containing this obj
        fileName
        
        % number of MT simulation replicates in this list.
        % nMT 50 = 50 simulation replicates.
        nMT
        
        % Model number as integer. Model 44.
        model
        
        % startParameter
        % contains simulation parameters
        %  .displayAll          1 = True, 0 = False. Normally set to 0.
        %  .startCondition      Integer, 1 = mimics a MT seed. Rest of MT empty.
        %  .eventMax            Integer, max # events in simulation. Normally 3x10^6.
        %  .timeMax             Integer, in seconds. 600 s here.
        %  .writeInterval - used to specify interval between writing simulation outputs.
        %                 - multiple conditions are possible. If one is satisfied, will
        %                 - write output.
        %                .writeIntervalEvent    Integer, 1000 here. 
        %                .writeIntervalTime     Float, 0.5 seconds here.
        %                .writeIntervalSize     Integer, 100 here. Max Size change btwn output.
        %  .stopIfShort         1 = T, 0 = F. Default 1.
        %  .washoutTime         timeMax + 1. prevents running forever. 601 s here.
        %  .deleteAllFiles      Delete Fortran outputs. 1 = T, 0 = F. Default 1. 
        startParameter
        
        % parameter = biochemical simulation parameters
        % .kon              On Rate constant, units per microMolar, per sec (uM-1, s-1)
        % .concent          Tubulin concentration, units Molar
        % Interface Affinities in units kT, input as negative values, like "-6.3"
        % .G_long_GTP       GTP Longitudinal Interaction Affinity  
        % .G_long_GDP       GDP Long. Interation Affinity 
        % .G_lat            Homotypic Lateral Interaction Affinity
        % .G_lat_seam       Heterotypic (at seam) Lateral Interaction Affinity
        % .G_bucket         Bucket Cost; "bucket" = subunit has lateral neighbor on each side
        % .G_bucket_seam    Bucket Cost at Seam; differentiates at seam if desired
        % .hyd              GTPase rate, units per second
        parameter
        
        % Raw simulation data
        data
        
        % Indvidual results from each simulation analysis as structure array
        % such as individual growth episodes
        stat
        
        % Results of aggregate analysis of the entire list
        % averages here
        listStat
        
        % Commandline used for calling the simulation program.
        % You can use this to run single simulation via command line.
        cmdLn
        
        % Flag to tell if simulations ran successfully or not.
        % 1 = T, 0 = F.
        simulationRan
        
        % Determines which (if any) extra analysis to do
        % Array of logicals. For example, if first element is true and
        % Second element is false, analysis 1 will be done but not analysis 2.
        % Regardless of the array, analysis 0 will always be done
        doAnalysis
        
        % Parameters for the analysis
        % indx = 1 is for analysis 0
        % indx = 2 is for analysis 1
        analysisPara
        
        % Used for model debugging. Causes simulation program to output
        % every event
        % displayAll
        
        % foldersize; it is also the size of work blocks
        folderSize
    end
    
    %% Constructor and default values 
    
    methods
        function obj = Smtl(filePath, fileName, model, startPara, nMT, parameter, doAnalysis, folderSize)
            
            % intitialize start parameter
            obj.startParameter = Smtl.getDefaultStartPara();
            
            % gets rid of filename extension
            [filepath,name,ext] = fileparts(fileName);
            fileName = name;
            
            % set obj properties
            if nargin >= 3
                obj.filePath = filePath;
                obj.fileName = fileName;
                obj.model = model;
            else
                error('insufficient input arguments')
            end
            if nargin >= 4
                if isstruct(startPara)
                    obj.startParameter = startPara;
                else
                    obj.startParameter = Smtl.getDefaultStartPara();
                    obj.startParameter.startCondition = startPara;
                end
            else
                obj.startParameter = Smtl.getDefaultStartPara();
            end
            if nargin >= 5
                obj.nMT = nMT;
            else
                obj.nMT = 1;
            end
            if nargin >= 6
                obj.parameter = parameter;
            else
                parameter = Smtl.getDefaultPara(model);
                obj.parameter = parameter;
            end
            if nargin >= 7
                obj.doAnalysis = doAnalysis;
            else
                obj.doAnalysis = Smtl.getDefaultAnalysis(model);
            end
            if nargin >= 8
                obj.folderSize = folderSize;
            else
                obj.folderSize = 100;
            end
            obj.simulationRan = false;

            % Determine the command string used to call simulation program
            obj = obj.setCmd();
            
            obj.save();
        end
    end
    
    % Default simulation parameter. Values have been set to those used in our
    % publication. Affinities are similar to the GTP values. Hydrolysis rate of 0.
    methods(Static)
        
        % Simulation Controls ----------------------
        % Can be changed at any time interactively or via command line.
        % example: "smtl.startPara.timeMax = 600;"
        function sp = getDefaultStartPara()
            sp = struct(...
                'displayAll', false,...         % can use true to debug.
                'startCondition', 1,...         % MT seed.
                'eventMax', 3e6,...             % max number of events.
                'timeMax', 600,...              % max desired runtime, seconds.
                'writeIntervalEvent', 1000,...  % max number of events between output.
                'writeIntervalTime', 0.5,...    % max time between events between output.
                'writeIntervalSize', 100,...    % max size change between output.
                'stopIfShort', true,...         % stop if simulation stops too early.
                'washoutTime', 601,...          % prevents running forever.
                'deleteAllFiles', true);        % deletes fortran outputs to save space.
        end
        
        % Simulation Parameters ----------------------
        % Can be changed at any time interactively or via command line.
        % example: "smtl.setPara(9).list = 0.01;" changes GTPase rate to 0.01 s-1
        % example: "smtl.setPara(9).list = [0:0.1:1]"; tests range of GTPase
        % list#: 1 = kon, 2 = concentration, 3 = G_long_GTP, 4 = G_long_GDP, 5 = G_lat
        %   6 = G_lat_seam, 7 = G_bucket, 8 = G_bucket_seam, 9 = hydrolysis
        function para = getDefaultPara(model)
            if model == 44
                para = struct('kon', 740959, 'concent', 12.5e-6, 'G_long_GTP', -6.4, 'G_long_GDP', -1.2, 'G_lat', -6.35, 'G_lat_seam', -6.35, 'G_bucket', 0, 'G_bucket_seam', 0, 'hyd', 0);
            end
        end
        
        % Ask program to perform two additional analyses
        function doAnalysis = getDefaultAnalysis(model)
            doAnalysis = false(20,1);
            if model == 44
                doAnalysis(8) = true;       % # and type of corner sites on MT end
                doAnalysis(4) = true;       % # GDP tubulins on MT end
            end
        end
        
        % Statistics created by the default analysis module.
        % this analysis is always done.
        function stat = getDefaultStat(doAnalysis)
            %default analysis
            stat.lifeTime = [];         % length of growth periods, in seconds. Specifically growtime of
                                        % MTs that end in catastrophe. Each growth episode = 1 lifetime. If MT
                                        % grows entire time with no catastrophe, will produce an empty array.
            stat.growthRate = [];       % in tubulins per second.
            stat.gWeight = [];          % stores timepoints of growth episodes.
            stat.shrinkingRate = [];    % in tubulins per second.
            stat.sWeight = [];          % stores timepoints of shrinking episodes.
            stat.grTaper = [];          % length of taper during growth; see fortran for details
            stat.grTaperStd = [];       % standard deviation
            stat.srTaper = [];          % length of taper during shrinking
            stat.grGTP = [];            % number of GTP-tubulins in MT during growth episodes.
            stat.srGTP = [];            % number of GTP-tubulins in MT during shrinking episodes.
            stat.gStart = [];           % timepoints when growth episodes start.
            stat.gEnd = [];             % timepoints when growth episodes end.
            stat.sStart = [];           % timepoints when shrinking episodes start.
            stat.sEnd = [];             % timepoints when shrinking episodes end.
            stat.nPause = [];           % number of pauses throughout entire simulation.
            stat.endsWithCata = [];     % 1 = T, 0 = F; does growth period end in catastrophe.
            
            % Corner Analysis
            if doAnalysis(4)
                stat.gCorner = [];      % mean # corners during growth.
                stat.gCornerStd = [];   % standard deviation.
                stat.sCorner = [];      % mean # corners during shrinking.
                stat.gTCorner = [];     % mean # true corners during growth.
                stat.gTCornerStd = [];  % standard deviation.
                stat.sTCorner = [];     % mean # true corners during shrinking.
            end
            
            % Terminal GDP Analysis
            if doAnalysis(8)
                stat.nTermGDP = [];     % mean # of GDP tubulin on MT End.
            end
        end
        
        % Initialize placeholders for data from the fortran outputs.
        function data = getDefaultData(model) 
            data.time = [];                 % time of simulation
            data.nTub = [];                 % # Tubulins in MT
            data.nGTP = [];                 % # GDP-tubulin at timepoints
            data.taper = [];                % std of heights of protofilaments
            if model == 44
                data.nTerminalGDP = [];     % # terminal GDP at timepoints
                data.corner = [];           % # corners at timepoints
                data.tCorner = [];          % # true corners at timepoints
            end
        end
        
        % Initialize placeholders for simulation statistics, calculated from data
        % Correspond to statistics above, but these are aggregate analyses of an Smtl
        function stat = getDefaultListStat(doAnalysis)
            
            %default analysis
            stat.n = [];
            stat.avgLT = [];
            stat.avgGR = [];
            stat.stdGR = [];
            stat.avgSR = [];
            stat.grTaper = [];
            stat.grTaperStd = [];
            stat.srTaper = [];
            stat.grGTP = [];
            stat.srGTP = [];
            stat.diffConst = [];
            stat.deltaL2 = [];
            stat.deltaT = [];
            stat.deltaW = [];
            
            stat.pauseFreq = [];
            stat.pauseToCataChance = [];

            % Corner Analysis
            if doAnalysis(4)
                stat.gCorner = [];
                stat.sCorner = [];
                stat.gTCorner = [];
                stat.sTCorner = [];
                stat.gCornerStd = [];
                stat.gTCornerStd = [];
            end
            
            % Terminal GDP Analysis
            if doAnalysis(8)
                stat.nTermGDP = [];
            end
        end
    end
    
    % Generates string to execute simulation on command line
    % Adds simulation controls
    methods
        function obj = setCmd(obj)
            model_ = round(obj.model);
            cmdLn_ = ['simulateMT_' num2str(model_) '.exe'];
            
            sp = obj.startParameter;
            cmdLn_ = [cmdLn_ ' ' num2str(length(fieldnames(sp)))];
            
            cmdLn_ = [cmdLn_ ' ' num2str(sp.displayAll)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.startCondition)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.eventMax)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.timeMax)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.writeIntervalEvent)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.writeIntervalTime)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.writeIntervalSize)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.stopIfShort)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.washoutTime)];
            cmdLn_ = [cmdLn_ ' ' num2str(sp.deleteAllFiles)];
            
            parameter_ = obj.parameter;
            if model_ == 44
                cmdLn_ = Smtl.addModelCmd_4(cmdLn_, model_, parameter_);
            end
            obj.cmdLn = cmdLn_;
        end
    end
    % Adds simulation parameters
    methods(Static)
        function cmdLn_ = addModelCmd_4(cmdLn_, model_, parameter_)
            if model_ == 44
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.kon)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.concent)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.G_long_GTP)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.G_long_GDP)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.G_lat)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.G_lat_seam)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.G_bucket)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.G_bucket_seam)];
                cmdLn_ = [cmdLn_ ' ' num2str(parameter_.hyd)];
            end
        end
    end

    % Default file and data formats
    methods(Static)
        % used by the basic analysis function to obtain file format
        % containing the simulation data
        function formatSpec = getFormatSpec(model)
            if model == 44
                formatSpec = '%10f%12f%12f%15f%15f%15f%f%[^\n\r]';                
            end
        end
        
        % used by basic analysis function to convert raw data cell into
        % data structure
        function data = convertData(rawData, model)
            data = struct();
            data.time = rawData{:, 1};              % simulation timepoints
            data.nTub = rawData{:, 2};              % # tubulins in MT
            data.nGTP = rawData{:, 3};              % number GTP-tubulins in MT
            data.taper = rawData{:, 4};             % taper data
            if model == 44
                data.nTerminalGDP = rawData{:, 5};  % number GDP-tub on MT end
                data.corner = rawData{:, 6};        % number corners on MT end
                data.tCorner = rawData{:, 7};       % number true corners on MT end
            end
        end
    end
    
    %% Run simulation of all MT and perform basic analysis.
    
    methods
        % Runs simulations and analyze; will not run if already analyzed.
        function obj = run(obj)
            folderSize_ = obj.folderSize;
            curDir = pwd;
            obj = obj.setCmd();
            
            % format path for unix and PC
            if ispc
                filePath_ = strrep (obj.filePath, '/', filesep);
                if ~exist([filePath_ obj.fileName], 'dir')
                    mkdir(filePath_, obj.fileName);
                end
                cmdLn_ = strrep (obj.cmdLn, '/', filesep);
                cmdLn_ = [cmdLn_, ' > "' obj.filePath obj.fileName filesep 'folder_'];
            elseif isunix
                filePath_ = strrep (obj.filePath, '\', filesep);
                if ~exist([filePath_ obj.fileName], 'dir')
                    mkdir(filePath_, obj.fileName);
                end
                cmdLn_ = strrep (obj.cmdLn, '\', filesep);
                cmdLn_ = ['./' cmdLn_, ' > "' obj.filePath obj.fileName filesep 'folder_'];
            end
            for i = 0:(obj.nMT-1)/folderSize_
                if ~exist([filePath_ obj.fileName filesep 'folder_' num2str(i)], 'dir')
                    mkdir([filePath_ obj.fileName filesep], ['folder_' num2str(i)]);
                end
                Smtl.createAnalysisDir([filePath_ obj.fileName filesep 'folder_' num2str(i)], obj.doAnalysis);
            end

            if ~obj.simulationRan
                fileName_ = obj.fileName;
                if isunix
                    cd(Smtl.modelLoc.unix);
                end
                parameter_ = obj.parameter;
                doAnalysis_ = obj.doAnalysis;
                stat_(obj.nMT) = Smtl.getDefaultStat(obj.doAnalysis);
                data_(obj.nMT) = Smtl.getDefaultData(obj.model);
                for c = 1:obj.nMT
                        fNum=floor((c-1)/folderSize_);
                        system([cmdLn_ num2str(fNum) filesep 'smt_' num2str(c) '.out"']);
                        [stat_(c), data_(c)] = Smtl.analyzeSingle(filePath_, [fileName_ filesep 'folder_' num2str(fNum)], c, doAnalysis_, parameter_, obj.startParameter, obj.model); %%#ok<AGROW>
                end
                if exist('stat_', 'var')
                    obj.stat = stat_;
                    obj.data = data_;
                    obj.listStat = obj.analyzeList();
                else
                    obj.listStat = [];
                end
                obj.simulationRan = true;
                obj.save();
                cd(curDir);
            end
        end
        
        
        % Only partially runs simulations.
        % That is to say if Smtl obj is suppose to run 1000 simulations,
        % this function will only run simulations in given folder
        % No analysis will be done, That has to be called separately
        %indx starts from 1
        function obj = partialRun(obj, indx)
            folderSize_ = obj.folderSize;
            curDir = pwd;
            obj = obj.setCmd();
            
            if ispc
                filePath_ = strrep (obj.filePath, '/', filesep);
                if ~exist([filePath_ obj.fileName], 'dir')
                    mkdir(filePath_, obj.fileName);
                end
                cmdLn_ = strrep (obj.cmdLn, '/', filesep);
                cmdLn_ = [cmdLn_, ' > "' obj.filePath obj.fileName filesep 'folder_'];
            elseif isunix
                filePath_ = strrep (obj.filePath, '\', filesep);
                if ~exist([filePath_ obj.fileName], 'dir')
                    mkdir(filePath_, obj.fileName);
                end
                cmdLn_ = strrep (obj.cmdLn, '\', filesep);
                cmdLn_ = ['./' cmdLn_, ' > "' obj.filePath obj.fileName filesep 'folder_'];
            end
            if ~exist([filePath_ obj.fileName filesep 'folder_' num2str(indx-1)], 'dir')
                mkdir([filePath_ obj.fileName filesep], ['folder_' num2str(indx-1)]);
            end
            
            Smtl.createAnalysisDir([filePath_ obj.fileName filesep 'folder_' num2str(indx-1)], obj.doAnalysis);
            
            
            if ~obj.simulationRan
                try
                    load([filePath_ fileName_ filesep 'folder_' num2str(indx-1) filesep], 'data');
                    analysisRan = true;
                catch
                    analysisRan = false;
                end
                
                if isunix
                    cd(Smtl.modelLoc.unix);
                end
                
                if analysisRan
                else
                    for c = ((indx-1)*folderSize_+1):min([indx*folderSize_, obj.nMT])
                        try
                            system([cmdLn_ num2str(indx-1) filesep 'smt_' num2str(c) '.out"']);
                        catch
                        end
                    end
                end
                cd(curDir);
            end
        end
        
        
        
        % Analysis to be done following partialRun()
        function [stat_, data_] = partialAnalyze(obj, indx)
            
            folderSize_ = obj.folderSize;
            obj = obj.setCmd();
            
            if ispc
                filePath_ = strrep (obj.filePath, '/', filesep);
            elseif isunix
                filePath_ = strrep (obj.filePath, '\', filesep);
            end
            fileName_ = obj.fileName;

            if ~obj.simulationRan
                % Try to load saved data/analysis
                try
                    partialAnalysis = load([filePath_ fileName_ filesep 'folder_' num2str(indx-1) filesep], 'data');
                    analysisRan = true;
                catch
                    analysisRan = false;
                end
                
                if analysisRan
                    stat_ = partialAnalysis.stat_;
                    data_ = partialAnalysis.data_;
                else
                    % Run partial anlysis
                    parameter_ = obj.parameter;
                    doAnalysis_ = obj.doAnalysis;
                    
                    endIndx = min([indx * folderSize_, obj.nMT]);
                    startIndxOffset = indx * folderSize_ - folderSize_;
                    chunkSize = endIndx - startIndxOffset;
                    
                    % Space allocation
                    stat_(chunkSize) = Smtl.getDefaultStat(obj.doAnalysis);
                    data_(chunkSize) = Smtl.getDefaultData(obj.model);

                    for c = 1:chunkSize
                        [stat_(c), data_(c)] = Smtl.analyzeSingle(filePath_, [fileName_ filesep 'folder_' num2str(indx-1)], c + startIndxOffset, doAnalysis_, parameter_, obj.startParameter, obj.model); %%#ok<AGROW>
                    end
                    
                    % Save progress
                    save([filePath_ fileName_ filesep 'folder_' num2str(indx-1) filesep 'data'], 'stat_', 'data_')
                end
            end
        end
        
        
    end
    
    %% Basic Analysis
    methods (Static)
        
        function [stat, data] = analyzeSingle(filePath, fileName, number, doAnalysis, parameter, startPara, model) %#ok<INUSL>
            
            stat=Smtl.getDefaultStat(doAnalysis);
            data=Smtl.getDefaultData(model);
            
            % Loading data from output file
            filename = [filePath filesep fileName filesep 'smt_' num2str(number) '.out'];
            startRow = 2;
            formatSpec = Smtl.getFormatSpec(model);
            
            % Try loading file. If unsuccessful, abort analysis
            try
                fileID = fopen(filename,'r');
            catch
                warning([fileName '_' num2str(number) ' could not open simulation output file']);
                return;
            end
            try
                dataArray = textscan(fileID, formatSpec, 'Delimiter', '', 'WhiteSpace', '', 'EmptyValue' ,NaN,'HeaderLines' ,startRow-1, 'ReturnOnError', false, 'CommentStyle', ' //');
            catch
                dataArray = textscan(fileID, formatSpec, 'Delimiter', '', 'WhiteSpace', '', 'EmptyValue' ,NaN,'HeaderLines' ,startRow-1, 'ReturnOnError', false, 'CommentStyle', '//');
            end
            fclose(fileID);

            data = Smtl.convertData(dataArray, model); 
            stat = Smtl.getDefaultStat(doAnalysis);
            
            saveLoc = [filePath filesep fileName filesep];
            saveLoc2 = [filesep 'smt_' num2str(number)];
            if true
                try
                    [stat] = Smtl.analysis_0(stat, data, [saveLoc 'analysis_0' saveLoc2]);
                catch
                end
            end
            
            % Corner Analysis
            if doAnalysis(4)
                try
                    [stat] = Smtl.analysis_4(stat, data);
                catch
                end
            end

            % Terminal GDP Analysis
            if doAnalysis(8)
                try
                    [stat] = Smtl.analysis_8(stat, data);
                catch
                end
            end
        end
        
        % Create necessary dirs to hold analysis outputs
        function createAnalysisDir(fp, AD)
            if ~exist([fp filesep 'analysis_0'], 'dir')
                mkdir(fp, 'analysis_0');
            end
            if AD(1)
                if ~exist([fp filesep 'analysis_1'], 'dir')
                    mkdir(fp, 'analysis_1');
                end
            end
            if AD(3)
                if ~exist([fp filesep 'analysis_3'], 'dir')
                    mkdir(fp, 'analysis_3');
                end
            end
        end
    end
    
    %% Analysis 0 - this is always done.
    methods(Static)
        % Determines the Growth Rate(s), Shrinking Rate(s), and Lifetime(s)
        % This section creates a figure that marks growth + shrinking episodes,
        % as well as catastrophe and rescue. Load this figure to check the program's
        % designation of these timepoints.
        function [stat] = analysis_0(stat, data, saveLoc)
            
            % initialize for LifeTime determination
            time = data.time;       % simulation writeout timepoints
            nTub = data.nTub;       % number tubulins in MT; integer.
            taper = data.taper;     % standard deviation in PF lengths; float.
            nGTP = data.nGTP;       % number GTP-tubulins in MT; integer.
            
            % Uses tracking of tubulins over time to determine when MT is
            % growing or shrinking. Also to determine catastrophe and rescue.
            maxTub = max(nTub);
            size = length(time);
            
            maxH = nTub(1);
            minH = 0;
            locTemp = 1;
            threshold = max([maxH- 200, maxH / 2]);
            
            isGrowing = true;            
            rescue = 1;
            cata = [];
            paused = [];
            
            thresholdDiff1 = 1500;  % 1000to check for catastrophe
            thresholdDiff2 = 50;    % 100check to see if shrinking paused for some reason
            thresholdDiff3 = 150;   % check for rescue
            
            figure;
            plot(time, nTub);
            xlabel('Simulation Time (sec)'); ylabel('Number tubulins in MT'); title('Tubulins in MT vs. Time (sec)');
            hold on;
            
            % Determine indices of rescue and catastrophe using previously-defined thresholds.
            for indx = 2:size
                height = nTub(indx);
                if isGrowing
                    if height >= maxH
                        maxH = height;
                        locTemp = indx;
                        threshold = max([maxH - thresholdDiff1, maxH / 2]);
                    
                    % Identify catastrophe
                    elseif height <= threshold
                        line([time(locTemp) time(locTemp)], [0 maxTub], [1 1], 'Color', 'r');
                        
                        cata(end+1) = locTemp; 
                        locTemp = indx;
                        
                        isGrowing = false;
                        isShrinking = true;
                        
                        minH = height;
                        threshold = min([minH + thresholdDiff2, minH * 2]);
                    end
                elseif isShrinking
                    if height <= minH
                        minH = height;
                        locTemp = indx;
                        threshold = min([minH + thresholdDiff2, minH * 2]);
                        
                    elseif height >= threshold
                        paused(end+1) = locTemp;
                        
                        threshold = min([minH + thresholdDiff3, minH * 2]);
                        
                        isShrinking = false;

                        % Identify Rescue
                        if height >= threshold
                            line([time(locTemp) time(locTemp)], [0 maxTub], [1 1], 'Color', 'c');
                            
                            rescue(end+1) = locTemp; 
                            locTemp = indx;
                            
                            isGrowing = true;
                            
                            maxH = height;
                            threshold = max([maxH - thresholdDiff1, maxH / 2]);
                        end
                    end
                else
                    if height <= minH
                        minH = height;
                        locTemp = indx;
                        threshold = min([minH + thresholdDiff3, minH * 2]);
                        
                    elseif height >= threshold
                        line([time(locTemp) time(locTemp)], [0 maxTub], [1 1], 'Color', 'c');
                        
                        rescue(end+1) = locTemp; 
                        locTemp = indx;
                        
                        isShrinking = false;
                        isGrowing = true;
                        
                        maxH = height;
                        threshold = max([maxH - thresholdDiff1, maxH / 2]);
                    end
                end
            end
            
            % Determine lifetime of MT
            stat.lifeTime = zeros(length(cata));
            for indx = 1:length(cata)
                stat.lifeTime(indx) = time(cata(indx)) - time(rescue(indx));
            end

            stat.lifeTime = stat.lifeTime';
            
            if isGrowing
                cata(end+1) = length(time);
            elseif isShrinking
                paused(end+1) = length(time);
            else
                rescue(end+1) = length(time);
            end
            grL = length(rescue);
            srL = length(cata);
            
            % initialize for growth rate and shrinking rate
            gr(grL) = 0;
            sr(srL) = 0;
            grW(grL) = 0; % weights
            srW(srL) = 0; % weights
            
            gCap(grL) = 0; % weight is same as grW
            sCap(grL) = 0; % weight is same as srW
            
            gTaper(grL) = 0;
            sTaper(grL) = 0;
            
            gStart(grL) = 0;
            gEnd(grL) = 0;
            sStart(srL) = 0;
            sEnd(srL) = 0;
            
            nPauses(grL) = 0;
            endsWithCatas(grL) = 0;
            
            fun = @(c,x) (x<c(1)) .* c(3) + (x>=c(1)&x<=c(2)) .* ((c(3)-c(4))/(c(1)-c(2)) .* x + (c(1)*c(4)-c(2)*c(3))/(c(1)-c(2))) + (x>c(2)) .* c(4);
            
            % Determine growth rate (GR), taper, nGTP, number of pauses.
            for indx = 1:grL
                try
                    % Determine the actual growth episode by fitting piecewise.
                    start = rescue(indx);
                    last = cata(indx);
                    
                    nTubAvg = (nTub(start)+ nTub(last)) / 2;

                    x0 = [time(start), time(last), nTub(start), nTub(last)];
                    ul = [time(last), time(last), nTubAvg, nTub(last)];
                    ll = [time(start), time(start), nTub(start), nTubAvg];
                    pfit = lsqcurvefit(fun, x0, time(start:last), nTub(start:last), ll, ul);
                    
                    % store fit
                    gStart(indx) = pfit(1);
                    gEnd(indx) = pfit(2);
                    
                    % draw detected growth region
                    line([pfit(1) pfit(2)], [pfit(3) pfit(4)], [1 1], 'Color', 'g');
                    
                    % calculate Growth Rate
                    gr(indx) = (pfit(3)-pfit(4)) / (pfit(1)-pfit(2));
                    grW(indx) = pfit(2) - pfit(1);
                    
                    % get index within the true growth region
                    grMask = (time >= pfit(1) & time <= pfit(2));
                    
                    % get GTP Cap
                    gCap(indx) = mean(nGTP(grMask));
                    
                    % get gTaper
                    gTaper(indx) = mean(taper(grMask));
                    gTaperStd(indx) = std(taper(grMask));
                    
                    % get nPauses-------------------------------------------
                    nPause = 0;
                    isPaused = false;
                    grThr = gr(indx) * 0.25;
                    for iData=start:(last-1)
                        curGR = (nTub(iData+1) - nTub(iData)) / (time(iData+1) - time(iData));
                        if isPaused
                            if curGR >= grThr
                                isPaused = false;
                            end
                        else
                            if curGR < grThr
                                nPause = nPause + 1;
                                isPaused = true;
                            end
                        end
                    end
                    
                    %last data point
                    if isPaused
                        if last ~= length(time)
                            endsWithCata = true;
                        else
                            endsWithCata = false;
                        end
                    else
                        if last ~= length(time)
                            endsWithCata = true;
                            nPause = nPause + 1;
                        else
                            endsWithCata = false;
                        end
                    end
                    
                    nPauses(indx) = nPause; 
                    endsWithCatas(indx) = endsWithCata; 
                catch
                    gr(indx) = 0;
                    grW(indx) = 0;
                    gCap(indx) = 0;
                    gTaper(indx) = 0;
                    gTaperStd(indx) = 0; 
                    nPauses(indx) = 0; 
                end
            end
            
            stat.gWeight = grW';
            stat.growthRate = gr';
            stat.grTaper = gTaper';
            stat.grTaperStd = gTaperStd';
            stat.grGTP = gCap';
            stat.gStart = gStart';
            stat.gEnd = gEnd';
            
            stat.nPause = nPauses';
            stat.endsWithCata = endsWithCatas';
            
            % determine Shrinking Rate (SR) ------------------------------------
            for indx = 1:srL
                try
                    start = cata(indx);
                    last = paused(indx);
                    
                    nTubAvg = (nTub(start)+ nTub(last)) / 2;

                    x0 = [time(start), time(last), nTub(start), nTub(last)];
                    ul = [time(last), time(last), nTub(start), nTubAvg];
                    ll = [time(start), time(start), nTubAvg, nTub(last)];
                    pfit = lsqcurvefit(fun, x0, time(start:last), nTub(start:last), ll, ul);

                    line([pfit(1) pfit(2)], [pfit(3) pfit(4)], [1 1], 'Color', 'm');

                    sr(indx) = (pfit(3)-pfit(4)) / (pfit(1)-pfit(2));
                    srW(indx) = pfit(2) - pfit(1);
                    
                    % store fit
                    sStart(indx) = pfit(1);
                    sEnd(indx) = pfit(2);
                    
                    % get index within the true gsr region
                    srMask = (time >= pfit(1) & time <= pfit(2));
                    
                    % get sCap
                    sCap(indx) = mean(nGTP(srMask));
                    
                    % get sTaper
                    sTaper(indx) = mean(taper(srMask));
                catch
                    sr(indx) = 0;
                    srW(indx) = 0;
                    sCap(indx) = 0;
                    sTaper(indx) = 0;
                end
            end
            
            stat.sWeight = srW';
            stat.shrinkingRate = sr';
            stat.srTaper = sTaper';
            stat.srGTP = sCap';
            stat.sStart = sStart';
            stat.sEnd = sEnd';
            
            % save figure; can use to check program's designation
            % of growth, shrinkage, rescue, catastrophe
            saveas(gcf, [saveLoc '.fig']);
            close(gcf);
        end
    end
    
    %% Analysis 4 - determines the average number of corner sites
    methods(Static)
        function [stat] = analysis_4(stat, data)
            % average # of corner while growing
            for indx_g = length(stat.gStart):-1:1
                mask = data.time >= stat.gStart(indx_g) & data.time <= stat.gEnd(indx_g);
                weight = sum(mask);
                gCorner(1,indx_g) = sum(data.corner(mask)) / weight;
                gTCorner(1,indx_g) = sum(data.tCorner(mask)) / weight;
                gCornerStd(1,indx_g) = std(data.corner(mask));
                gTCornerStd(1,indx_g) = std(data.tCorner(mask));
            end
            
            % average # of corner while shrinking
            for indx_s = length(stat.sStart):-1:1
                mask = data.time >= stat.sStart(indx_s) & data.time <= stat.sEnd(indx_s);
                weight = sum(mask);
                sCorner(1,indx_s) = sum(data.corner(mask)) / weight;
                sTCorner(1,indx_s) = sum(data.tCorner(mask)) / weight;
            end
            
            stat.gCorner = gCorner;
            stat.sCorner = sCorner;
            stat.gTCorner = gTCorner;
            stat.sTCorner = sTCorner;
            stat.gCornerStd = gCornerStd;
            stat.gTCornerStd = gTCornerStd;
        end
    end
    
    %% Analysis 8 - terminal GDP analysis
    methods(Static)
        function [stat] = analysis_8(stat, data)
            for i = length(stat.gStart):-1:1
                try
                    nTermGDP(i) = nanmean(data.nTerminalGDP(data.time >= stat.gStart(i) & data.time <= stat.gEnd(i)));
                    ntg_weight(i) = stat.gEnd(i) - stat.gStart(i);
                catch
                    nTermGDP(i) = 0;
                    ntg_weight(i) = 0;
                end
            end
            stat.nTermGDP = nTermGDP .* ntg_weight / nansum(ntg_weight);
        end
    end
    
    
    %% Aggregate Analysis

    methods
        function stat_ = analyzeList(obj)
            
            stat_ = Smtl.getDefaultListStat(obj.doAnalysis);

            if true
                nMTs = 0; %also calculate number of MTs
                totLT = 0;
                totCat = 0;
                for indx = obj.nMT:-1:1 %transfer only the first element
                    nMTs = nMTs + length(obj.stat(indx).lifeTime);
                    try
                        if isempty(obj.stat(indx).lifeTime)
                            totLT = totLT + obj.data(indx).time(end);
                        else
                            totLT = totLT + obj.stat(indx).lifeTime(1);
                            totCat = totCat + 1;
                        end
                        lt(indx) = obj.stat(indx).lifeTime(1);
                    catch
                        lt(indx) = nan;
                    end
                    try
                        grW(indx) = obj.stat(indx).gWeight(1);
                        gr(indx) = obj.stat(indx).growthRate(1);
                        grTaper(indx) = obj.stat(indx).grTaper(1);
                        grTaperStd(indx) = obj.stat(indx).grTaperStd(1);
                        grGTP(indx) = obj.stat(indx).grGTP(1);
                        nPause(indx) = obj.stat(indx).nPause(1);
                        nCata(indx) = obj.stat(indx).endsWithCata(1);
                        if isnan(grW(indx)) || isnan(gr(indx)) || isnan(grTaper(indx)) || isnan(grGTP(indx))
                            grW(indx) = 0;
                            gr(indx) = 0;
                            grTaper(indx) = 0;
                            grGTP(indx) = 0;
                            nPause(indx) = 0;
                            nCata(indx) = 0;
                        end
                    catch
                        grW(indx) = 0;
                        gr(indx) = 0;
                        grTaper(indx) = 0;
                        grGTP(indx) = 0;
                        nPause(indx) = 0;
                        nCata(indx) = 0;
                    end
                    try
                        srW(indx) = obj.stat(indx).sWeight(1);
                        sr(indx) = obj.stat(indx).shrinkingRate(1);
                        srTaper(indx) = obj.stat(indx).srTaper(1);
                        srGTP(indx) = obj.stat(indx).srGTP(1);
                        if isnan(srW(indx)) || isnan(sr(indx)) || isnan(srTaper(indx)) || isnan(srGTP(indx))
                            srW(indx) = 0;
                            sr(indx) = 0;
                            srTaper(indx) = 0;
                            srGTP(indx) = 0;
                        end
                    catch
                        srW(indx) = 0;
                        sr(indx) = 0;
                        srTaper(indx) = 0;
                        srGTP(indx) = 0;
                    end
                end
                
                % store n
                stat_.n = nMTs;
                
                % calcualate normalization factor
                grWTot = sum(grW);
                srWTot = sum(srW);
                
                % get avg
                stat_.avgGR = sum(gr.*grW) / grWTot;
                stat_.avgSR = sum(sr.*srW) / srWTot;
                stat_.stdGR = std(gr,grW);
                
                stat_.grTaper = sum(grTaper.*grW) / grWTot;
                stat_.grTaperStd = sum(grTaperStd.*grW) / grWTot;
                stat_.srTaper = sum(srTaper.*srW) / srWTot;
                
                stat_.grGTP = sum(grGTP.*grW) / grWTot;
                stat_.srGTP = sum(srGTP.*srW) / srWTot;
                
                stat_.avgLT = totLT / totCat;
                
                stat_.pauseFreq = sum(nPause) / grWTot;
                stat_.pauseToCataChance = sum(nCata) / sum(nPause);
                
                % getDiffusionConstant------------------------------------------------------
                % set max delta T
                max_dt = 20;
                tDiffTot = [];
                lDiff2Tot = [];
                
                % for determining the bin size
                n_tStepSize = 0;
                tStepSize = 0;
                               
                for indx = obj.nMT:-1:1 % transfer only the first element
                    noise = zeros(1,7498);

                    try
                        
                        
                        if length(obj.stat(indx).gEnd) >= 1
                            for indx_gr = 1:length(obj.stat(indx).gEnd)
                                time = obj.data(indx).time;
                                if obj.stat(indx).growthRate(indx_gr) > min([0, 4 * stat_.avgGR]) && obj.stat(indx).growthRate(indx_gr) < max([4 * stat_.avgGR, 0])
                                    mask = time >= obj.stat(indx).gStart(indx_gr) & time <= obj.stat(indx).gEnd(indx_gr) & time > 10;
                                    time = time(mask);
                                    nTub = obj.data(indx).nTub(mask);

                                    % aggregate all delta_l^2 and delta_t data
                                    indx_l2 = 0;
                                    for indx_time = 2:length(time)
                                        for indx_time2 = 1:(indx_time-1)
                                            if time(indx_time) - time(indx_time2) > max_dt
                                                continue;
                                            end
                                            indx_l2 = indx_l2 + 1;
                                            tDiff(indx_l2) = time(indx_time) - time(indx_time2);
                                            lDiff2(indx_l2) = (nTub(indx_time)+noise(mod(floor(time(indx_time)*10),7498)+1) - nTub(indx_time2)-noise(mod(floor(time(indx_time2)*10),7498)+1) - stat_.avgGR * tDiff(indx_l2)).^2;
                                        end
                                    end
                                    if ~isempty(time)
                                        n_tStepSize = n_tStepSize + length(time) - 1;
                                        tStepSize = tStepSize + time(end) - time(1);

                                        tDiffTot = [tDiffTot, tDiff(1:indx_l2)]; 
                                        lDiff2Tot = [lDiff2Tot, lDiff2(1:indx_l2)]; 
                                    end
                                end
                            end
                        end
                    catch 
                    end
                    if obj.startParameter.startCondition == 3
                        obj.stat(indx).gStart = temp_gStart;
                        obj.stat(indx).gEnd = temp_gEnd;
                    end
                end
                
                
                binSize = tStepSize / n_tStepSize * 2;
                nBin = max([floor(max_dt/binSize)-1, 20]);
                tDiffAvg(nBin)=0;
                lDiffAvg(nBin)=0;
                nDiffAvg(nBin)=0;
                
                for i=1:nBin
                    tempL2 = lDiff2Tot(tDiffTot < i*binSize & tDiffTot > (i-1)*binSize);
                    tempL2 = sort(tempL2);
                    t_listlength = length(tempL2);
                    if t_listlength > 0
                        temp_threshold = floor(t_listlength/20);
                        tempL2((t_listlength-temp_threshold):t_listlength) = [];
                        tempL2(1:temp_threshold) = [];
                        tDiffAvg(i) = nanmean(tDiffTot(tDiffTot < i*binSize & tDiffTot > (i-1)*binSize));
                        lDiffAvg(i) = nanmean(tempL2);
                        nDiffAvg(i) = length(tempL2);
                    else
                        tDiffAvg(i) = nan;
                        lDiffAvg(i) = nan;
                        nDiffAvg(i) = nan;
                    end
                end
                nanIndx = isnan(tDiffAvg) | isnan(lDiffAvg);
                tDiffAvg(nanIndx) = [];
                lDiffAvg(nanIndx) = [];
                nDiffAvg(nanIndx) = [];
                
                stat_.deltaL2 = lDiffAvg;
                stat_.deltaT = tDiffAvg;
                stat_.deltaW = nDiffAvg;
                
                lDiffAvg = lDiffAvg(2:end);
                tDiffAvg = tDiffAvg(2:end);
                
                % remove time points that are too small or too big
                mask = tDiffAvg>5;
                lDiffAvg = lDiffAvg(mask);
                tDiffAvg = tDiffAvg(mask);
                mask = tDiffAvg<40;
                lDiffAvg = lDiffAvg(mask);
                tDiffAvg = tDiffAvg(mask);
                
                if length(lDiffAvg)>2
                    try
                        pfit = fit(tDiffAvg', lDiffAvg', 'poly1', 'Lower', [0, -inf], 'Upper', [inf, inf]);
                        stat_.diffConst = pfit.p1 / 2;
                    catch
                        stat_.diffConst = nan;
                    end
                else
                    stat_.diffConst = nan;
                end
                
            end
            
            % Corner Analysis
            if obj.doAnalysis(4)
                stat_.gCorner = nanmean([obj.stat.gCorner]);
                stat_.gCornerStd = nanmean([obj.stat.gCornerStd]);
                stat_.sCorner = nanmean([obj.stat.sCorner]);
                stat_.gTCorner = nanmean([obj.stat.gTCorner]);
                stat_.gTCornerStd = nanmean([obj.stat.gTCornerStd]);
                stat_.sTCorner = nanmean([obj.stat.sTCorner]);
            end
            
            % Terminal GDP-tubulin analysis
            if obj.doAnalysis(8)
                stat_.nTermGDP = nanmean([obj.stat.nTermGDP]);
            end
        end
    end
    
    %% Delete files
    %  the output of the simulation takes up quite a bit of memory
    %  This function can be use to delete these no longer used files to free
    %  up some memory
    
    methods
        
        function deleteSimOut(obj)
            if ispc
                filePath_ = strrep (obj.filePath, '/', filesep);
            elseif isunix
                filePath_ = strrep (obj.filePath, '\', filesep);
            end
            for folder = 0:(obj.nMT-1)/obj.folderSize
                indxStart = folder*obj.folderSize;
                for indx = (indxStart + 1):min([indxStart + obj.folderSize, obj.nMT])
                    delete([filePath_ filesep obj.fileName filesep 'folder_' num2str(folder) filesep 'smt_' num2str(indx) '.out']);
                end
            end
        end
        
        function deleteSimOut2(obj)
            if ispc
                filePath_ = strrep (obj.filePath, '/', filesep);
            elseif isunix
                filePath_ = strrep (obj.filePath, '\', filesep);
            end
            for folder = 0:(obj.nMT-1)/obj.folderSize
                indxStart = folder*obj.folderSize;
                for indx = (indxStart + 1):min([indxStart + obj.folderSize, obj.nMT])
                    f = [filePath_ filesep obj.fileName filesep 'folder_' num2str(folder) filesep 'smt_' num2str(indx) '.out'];
                    if exist(f, 'file')
                        delete(f);
                    end
                end
                
                f = [filePath_ filesep obj.fileName filesep 'folder_' num2str(folder) filesep 'data.mat'];
                if exist(f, 'file')
                    delete(f);
                end
            end
        end
        
    end
    
    
    %% save and load
    methods
        function save(smtl)
            if ~exist(smtl.filePath, 'dir')
                mkdir(smtl.filePath);
            end
            save([smtl.filePath smtl.fileName], 'smtl');
        end
    end
    
    methods(Static)
        function obj = load(path, name)
            load([path name]);
            obj=smtl;
        end
        
        function obj = uiload()
            [name, path] = uigetfile();
            obj = Smtl.load(path, name);
        end
    end
    
end