classdef Smtll
    % Cleary et al. 2022, Elife
    % Smtll - Simulated Microtubule List List
    %
    % Creates, Runs, Analyses List of Smtl 
    %
    % Stores information about starting parameters and aggregate results
    % from MT simulations. This class allows easy generation of lists of
    % Smtl to run, and facilitates aggregate analysis of simulation data.
    % 
    %
    % All Smtl within the list must have identical model, startCondition,
    % and analysisDone (analogous to Smtl.doAnalysis).
    % 
    % Abbreviations:
    %   MT = microtubule
    %   Smtl = Simulate Microtubule List
    %   Smtll = Simulate Microtubule List List
    %
    % Matlab Version - 2019a
    %
    % Red Hat Enterprise Linux Server 7.6 (Maipo)
    
    %% Class Variables
    properties (Constant)
        folderSize = 200;
    end
    
    properties
        
        % Directory where this object (Smtll) is stored; input string
        filePath

        % and the file name, input as string, give name without an extension
        fileName
        
        % Simulation Model #, integer.'44'
        model
        
        % Analysis Modules and Parameters
        analysisDone 
        
        %list of parameters, (analyzed) statistics, and file locations of
        % each Smtl in the Smtll
        startPara   % Simulation parameters (time, write interval, etc.)
        para        % Model Parameters for each Smtl (kon, concentration, etc.)
        nMT         % Integer; # of repeated simulations in Smtl = # MT simulations
        stat        % Holds aggregate analysis of Smtl
        floc        % File locations of Smtl output
        simRan      % 1 = yes, 0 = no; Reports for each Smtl.
                    % If there is an issue with model output format, this will report "0",
                    % Because Smtl (and thus Smtll) won't be able to parse it.
        data        % cell of more specific statistics of individual simulations
                    % "data" is no longer used to reduce file size
                    % better to just get smtl using getSmtl function
        
        % list length; Integer, corresponds to number of Smtl
        listLength
        
        % setPara
        % Setup Parameter and nMT: these variables can be used to quickly
        % create and add set of smtl
        %
        % setPara holds values to generate lists of parameters using
        % intervals and list
        %
        % .name  = parameter condition name
        % .group = must be non_neg integer
        %       = '0' no group = when group number is identical, values of
        %       parameters change together. When group# is the same, the
        %       total length of the final list of numbers must also be the
        %       same. (It is good idea to set all unchanging values to one
        %       group that is not 0)
        % .start = starting value
        % .end   = end value. Cannot equal to .start, unless .nInt = 0,1
        % .nInt  = number of values to be used over interval.
        %       = '0' will not generate any interval values. .list must
        %       contain at least 1 value
        %       = '1' only start value will be used
        % .list  = list number additional values to be used. Will be added
        %       after the generated list from above (ie.[start:x:end list])
        %
        % nMT = number repeated simulation under same Smtl
        setPara

        % Number of repeated MT simulations in same Smtl; Integer
        nMTGen

        % Number of times addGenerated() is called
        iGen

        % Run() statistics - timestamp, nWorkers, analysis time
        compStat
        
        % smtl folderSize
        smtlFolderSize
        
        % slurmJob ID if using slurm queue
        slurmJob
    end
    

    %% Constructor
    methods
        % create the Smtll.
        % example: x = Smtll('/example/','testRun',44,1,50); 
        %   dirname = /example/, filename = testRun, model = 44, startCondition = 1, nMTGen = 50 MT simulations. 
        %   no specifiers for analysisDone and smtlFolderSize = uses default values.
        function obj = Smtll(filePath, fileName, model, startCondition, nMTGen, analysisDone, smtlFolderSize)
            
            % intitialize start parameter for each Smtl
            obj.startPara = Smtl.getDefaultStartPara();
            
            % Move inputs into obj.properties
            % or use default values
            if nargin >= 3
                
                %gets rid of extension if one is given in filename
                [filepath,name,ext] = fileparts(fileName);
                fileName = name;

                % adds filesep to filePath if it doesn't already exist
                if filePath(end) ~= filesep
                    filePath(end+1) = filesep;
                end
                obj.filePath = filePath;
                obj.fileName = fileName;
                obj.model = model;
            else
                obj.filePath = '';
                obj.fileName = '';
                obj.model = 44;
                model = 44;
            end
            
            % Move inputs into obj.properties
            % or use default values
            if nargin >= 4
                obj.startPara.startCondition = startCondition;
            else
            end
            if nargin >= 5
                obj.nMTGen = nMTGen;
            else
                obj.nMTGen = 10;
            end
            if nargin >= 6
                obj.analysisDone = analysisDone;
            else
                obj.analysisDone = Smtl.getDefaultAnalysis(model);
            end
            if nargin >= 7
                obj.smtlFolderSize = smtlFolderSize;
            else
                obj.smtlFolderSize = 100;
            end
            obj.listLength = 0;
            obj.iGen = 0;
            
            %initialize structure array para and floc
            obj.para = Smtl.getDefaultPara(model);
            obj.para(1) = [];
            obj.floc = struct('path', '', 'name', '');
            obj.stat = [];
            obj.compStat = struct('timeStamp',[],'nWorkers',[],'simTime',[],'analysisTime1',[], 'analysisTime2',[]);
            
            %Set up paraInter
            % obtain default parameter to obtain structure of a typical parameter
            defPara = Smtl.getDefaultPara(model);
            
            % get field names of the parameter
            fnames = fieldnames(defPara);
            
            % Transfer to paraInter and initialize setPara
            setPara_(length(fnames)) = struct('name', [], 'group', [], 'start', [], 'end', [], 'nInt', [], 'list', []);
            for indx = 1:length(fnames)
                setPara_(indx).name = fnames{indx};
                setPara_(indx).group = 0;
                setPara_(indx).start = defPara.(fnames{indx});
                setPara_(indx).end = setPara_(indx).start;
                setPara_(indx).nInt = 0;
                setPara_(indx).list = setPara_(indx).start;
            end
            obj.setPara = setPara_;
            obj.slurmJob = nan;
        end
    end
    
    %% Add and Remove
    methods
        
        % adds Smtl to existing Smtll
        function obj = add(obj, smtl)
            if smtl.model == obj.model
                if smtl.startParameter.startCondition == obj.startPara.startCondition
                    if all(obj.analysisDone == smtl.doAnalysis)
                        obj.listLength = obj.listLength + 1;
                        obj.para(obj.listLength) = smtl.parameter;
                        obj.floc(obj.listLength) = struct('path', smtl.filePath, 'name', smtl.fileName);
                        obj.nMT(obj.listLength,1) = smtl.nMT;
                        obj.data{obj.listLength,1} = [];
                        if smtl.simulationRan
                            if ~isstruct(obj.stat)
                                obj.stat = smtl.listStat;
                            end
                            obj.data{obj.listLength,1} = smtl.stat;
                            obj.stat(obj.listLength) = smtl.listStat;
                        end
                        obj.simRan(obj.listLength,1) = smtl.simulationRan;
                    else
                        warning('different set of analysis has been done')
                    end
                else
                    warning('simulation start conditions do not match')
                end
            else
                warning('simulation models do not match');
            end
        end
        
        % creates and add Smtl
        function obj = addNew(obj, name, nMT, para, sp)
            smtl = Smtl([obj.filePath obj.fileName filesep 'folder_' num2str(floor((obj.iGen-1)/Smtll.folderSize)) filesep], name, obj.model, sp, nMT, para, obj.analysisDone, obj.smtlFolderSize);
            smtl.startParameter = sp;
            smtl.save;
            obj = obj.add(smtl);
        end
        
        % Run this after creating and editing Smtll as desired
        % generates new Smtl using setPara and adds to the this
        %
        % For group number:
        % 0 - Changes first
        % N
        % N-1
        % ...
        % 2
        % 1 - Changes last
        function obj = addGenerated(obj)
            
            %convert all obj groupN into integer
            paraL = length(obj.setPara);
            for indx = 1:paraL
                obj.setPara(indx).group = floor(obj.setPara(indx).group);
            end
            
            %generate list of parameter values for each parameter
            paraL = length(obj.setPara);
            finList{paraL} = [];
            for indx = 1:paraL
                if obj.setPara(indx).nInt == 0
                    finList{indx} = [];
                else
                    startV = obj.setPara(indx).start;
                    endV = obj.setPara(indx).end;
                    finList{indx} = linspace(startV, endV, obj.setPara(indx).nInt);
                end
                finList{indx} = [finList{indx} obj.setPara(indx).list];
                if isempty(finList{indx})
                    error(['Not enough values for ' obj.setPara(indx).name]);
                end
            end
            
            % determine largest group #
            maxG = max([obj.setPara.group]);
            
            % Check to see if every memeber of the group have same value
            % List length and assign parameters to groups
            % First check all group number is non_neg integer
            groupN = [obj.setPara.group];
            if any(groupN < 0)
                error('group number must be a non_negative integer')
            end
            
            % Assign parameters to groups
            group0 = find([obj.setPara.group] == 0);
            if maxG > 0
                group{maxG} = [];
                for indx = 1:maxG
                    group{indx} = find([obj.setPara.group] == indx);
                end

                % Check to see if group contains any parameters; if not discard
                % Also, make sure group members have value lists same length
                for indx = maxG:-1:1
                    if isempty(group{indx})
                        %Discard
                        group(indx) = [];
                        maxG  = maxG - 1;
                    elseif length(group{indx}) == 1
                        %There's only one, so no need to compare
                    else
                        %comparing list lengths to the list length of first list
                        listL = length(finList{group{indx}(1)});
                        for indx2 = 2:length(group{indx})
                            if listL ~= length(finList{group{indx}(indx2)})
                                error([obj.setPara(group{indx}(indx2)).name ' has parameter value list with different length']);
                            end
                        end
                    end
                end
            end
            
            % Add 'group0' to the end of 'group'
            if ~isempty(group0)
                group{maxG + length(group0)} = [];
                for indx = 1:length(group0)
                    group{indx + maxG} = group0(indx);
                end
                maxG = maxG + length(group0);
            end
            
            % generate naming scheme
            names{maxG} = [];
            for indx = 1:maxG
                names{indx} = obj.setPara(group{indx}(1)).name(1:3);
            end
            
            
            % Generate counter (and its maximum)
            counter = ones(1, maxG);
            countMax = counter;
            for indx = 1:maxG
                countMax(indx) = length(finList{group{indx}(1)});
            end
            
            parameter = zeros(1,paraL);
            while counter(1) <= countMax(1)
                
                % increment the counter iGen
                obj.iGen = obj.iGen + 1;
                
                % determine parameter and name
                name = [obj.fileName '_' num2str(obj.iGen)];
                for indx = 1:maxG
                    for indx2 = 1:length(group{indx})
                        paraIndx = group{indx}(indx2); %nth parameter in the setPara list
                        parameter(paraIndx)= finList{paraIndx}(counter(indx));
                    end
                end
                
                % convert list of parameter into structure
                for indx = 1:paraL
                    para_.(obj.setPara(indx).name) = parameter(indx);
                end
                
                % add new Smtl
                obj = obj.addNew(name, obj.nMTGen, para_, obj.startPara);
                
                % count up
                counter(maxG) = counter(maxG) + 1;
                if counter(maxG) > countMax(maxG) && maxG ~=1
                    counter(maxG) = 1;
                    counter(maxG-1) = counter(maxG-1) + 1;
                    for indx = (maxG-1):-1:2
                        if counter(indx) > countMax(indx)
                            counter(indx) = 1;
                            counter(indx-1) = counter(indx-1) + 1;
                        else
                            break;
                        end
                    end
                end
            end
            
            % save obj
            obj.save;
            
        end
        
        % removes Smtl
        function obj = remove(obj, indxList)
            indxList(indxList < 1 | indxList > length(obj.para)) = [];
            if indxList <= length(obj.para)
                obj.para(indxList) = [];
                indxList = sort(indxList,'descend');
                for indx = indxList
                    try
                        obj.stat(indx) = [];
                    catch
                    end
                end
                obj.floc(indxList) = [];
                obj.simRan(indxList) = [];
                obj.data(indxList) = [];
                obj.nMT(indxList) = [];
                obj.listLength = obj.listLength - length(indxList);
            else
                warning('Index out of bounds')
            end
        end
        
        % removes all Smtl
        function obj = removeAll(obj)
            if ~isempty(obj.para)
                obj = obj.remove(1:length(obj.para));
            end
        end
    end
    
    %% Run simulation
    
    methods
        % Runs the simulation if not already run and do the Smtl analysis
        % and collect the analysis data
        % Lastly erases all simulation output files to save space
        function obj = run(obj)
            obj.compStat.timeStamp = clock;%
            obj.compStat.nWorkers = gcp;%
            
            % Load smtl objs; If that fails, removes them from the list
            fileLoc = obj.floc;
            for indx = obj.listLength:-1:1
                try
                    smtllCell{indx} = Smtl.load(fileLoc(indx).path, [fileLoc(indx).name '.mat']);
                catch
                    smtllCell(indx)=[];
                    obj=obj.remove(indx);
                end
            end
            for indx = 1:obj.listLength
                smtll(indx) = smtllCell{indx}; 
            end

            
            % Preallocation
            nFolder = ceil(obj.nMT / obj.smtlFolderSize);
            partialSize = sum(nFolder(~obj.simRan));
            partialList(partialSize) = struct('indxSmtl', [], 'indxFolder', []);
            partialIndxStart(length(obj.listLength)) = 0;
            indx = 1;
            smtllCell{partialSize} = [];
            
            for indxSmtl = 1:obj.listLength
                partialIndxStart(indxSmtl) = indx;
                for indxFolder = 1:(nFolder(indxSmtl)*~obj.simRan(indxSmtl))
                    partialList(indx).indxSmtl = indxSmtl;
                    partialList(indx).indxFolder = indxFolder;
                    smtllCell{indx} = smtll(indxSmtl);
                    indx = indx + 1;
                end
            end
            s{partialSize} = [];
            d{partialSize} = [];
            tic;
            
            parfor indx = 1:partialSize
                try
                    smtl = smtllCell{indx};
                    smtl.partialRun(partialList(indx).indxFolder);
                catch
                end
            end
            
            obj.compStat.simTime = toc/3600;%
            tic;
            
            % Analysis Single           
            AD = obj.analysisDone;
            
            parfor indx = 1:partialSize
                try
                    smtl = smtllCell{indx};
                    [s{indx}, d{indx}] = smtl.partialAnalyze(partialList(indx).indxFolder);
                catch
                end
            end
            
            obj.compStat.analysisTime1 = toc/3600;
            tic
            
            % Combine Smtls and do List Analysis
            delete = false(1,obj.listLength);
            for indxSmtl = 1:obj.listLength
                try
                    if ~obj.simRan(indxSmtl)
                        indx = partialIndxStart(indxSmtl);
                        s_temp = s{indx};
                        d_temp = d{indx};
                        for indx = partialIndxStart(indxSmtl)+(1:nFolder(indxSmtl)-1)
                            s_temp = [s_temp s{indx}]; 
                            d_temp = [d_temp d{indx}]; 
                        end
                        smtll(indxSmtl).stat = s_temp;
                        smtll(indxSmtl).data = d_temp;
                        smtll(indxSmtl).listStat = smtll(indxSmtl).analyzeList;
                        smtll(indxSmtl).simulationRan = true;
                        smtll(indxSmtl).save;
                    end
                catch
                    delete(indxSmtl)=true;
                end
                
            end
            
            obj.compStat.analysisTime2 = toc/3600;
            
            % Delete raw simulation output and folders
            parfor indx = 1:obj.listLength
                try
                    if ~obj.simRan(indx) 
                        smtll(indx).deleteSimOut2();
                    end
                catch
                end
            end
            
            % Transfer data to smtll
            obj.stat = Smtl.getDefaultListStat(AD);
            obj.stat(obj.listLength) = Smtl.getDefaultListStat(AD);
            for indx = 1:obj.listLength
                try
                    if ~obj.simRan(indxSmtl)
                        obj.stat(indx) = smtll(indx).listStat;
                        obj.simRan(indx) = smtll(indx).simulationRan;
                    end
                catch
                    obj.stat(indx) = Smtl.getDefaultListStat(AD);
                    obj.simRan(indx) = false;
                end
            end
            
            % save
            obj.save;
        end
    end
  
    %% Show Figure created by Smtl
    methods
        function showFig(obj, iSmtl, iMT)
            % iSmtl - index number of the smtl within smtll obj
            % iMT - index number of the simulation within the smtl
            % figure automatically created; stored in analysis_0 subdir
            fileLoc = [obj.floc(iSmtl).path, obj.floc(iSmtl).name, filesep];
            folderNum = floor((iMT-1) / obj.smtlFolderSize);
            
            fileLoc = [fileLoc 'folder_' num2str(folderNum) filesep];
            fileLoc = [fileLoc 'analysis_0' filesep 'smt_' num2str(iMT) '.fig'];
            
            openfig(fileLoc);
        end
        
        % Plots deltaT and deltaL2 for the Smtl
        function showDeltaL2(obj, iSmtl)
            figure;
            plot(obj.stat(iSmtl).deltaT, obj.stat(iSmtl).deltaL2);
            xlabel('deltaT'); ylabel('deltaL2');
        end
    end
    
    %% Get Smtl - Load an Smtl (Simulate MicroTubule List) created by Smtll
    %% With Smtll loaded, type "smtll.getSmtl(n)" to load Smtl (n)
    methods
        function [smtl] = getSmtl(obj, iSmtl)
            smtl = Smtl.load(obj.floc(iSmtl).path, obj.floc(iSmtl).name);
        end
    end
    
    %% Save and Load Current Smtll Object or Load Previous Smtll Object
    
    methods
        function save(smtll)
            if ~exist(smtll.filePath, 'dir')
                mkdir(smtll.filePath);
            end
            smtll.slurmJob = nan;
            save([smtll.filePath smtll.fileName], 'smtll');
        end
    end
    
    methods(Static)
        function obj = load(path, name)
            load([path name]);
            obj=smtll;
        end
        
        function obj = uiload()
            [name, path] = uigetfile();
            obj = Smtll.load(path, name);
        end
    end
    
    methods
        function obj = reload(obj)
            job = obj.slurmJob;
            obj=Smtll.load(obj.filePath, obj.fileName);
            obj.slurmJob = job;
        end
    end
    
    
end

