program simulateMT
    implicit none

    !Developed by the Rice Lab at UT Southwestern Medical Center
    !to compile, we used this module: module load intel/15.0.0
    !Red Hat Enterprise Linux Server 7.6 (Maipo)
    !
    !To compile this program:
    !   ifort -extend-source 132 -o simulateMT_44.exe simulateMT_44.f90
    !
    !To run this program from command line (Example values):
    !   ./simulateMT_44.exe 10 0 1 3000000 600 1000 0.5 100 1 601 1 740959 1.5e-06 -9.2 -1.6 -8.2 0 0 0 > test.out
    !
    !       simulation controls: 10 0 1 3000000 600 1000 0.5 100 1 601 1
    !                            10 = number of control parameters
    !                             0 = displayAll to False; 1 = True
    !                             1 = start condition, MT seed
    !                       3000000 = max number of events in simulation
    !                           600 = max simulation time in seconds.
    !                  1000 0.5 100 = write interval sizes: either 1000 events, 0.5 seconds, or 100 size
    !                             1 = stop if Short, 1 = True
    !                           601 = washout time is time max + 1; keeps from running foever
    !                             1 = deleteAllFiles, 1 = True; prevents buildup of many small files
    !
    !       simulation parameters: 740959 1.5e-06 -9.2 -1.6 -8.2 0 0 0
    !                        740959 = on rate constant, per uM, per sec
    !                        1.5e-6 = Tubulin concentration
    !                          -9.2 = deltaG Longitudinal, GTP; kT
    !                          -1.6 = deltaG Longitudinal, GDP; kT
    !                          -8.2 = deltaG Lateral, kT
    !                             0 = Bucket cost, non-seam; kT
    !                             0 = Bucket cost, seam; kT
    !                             0 = GTPase rate; units per second
    !   
    !
    !Microtubule (MT) Matrix Description
    !   max rows (y) = 20000;   tubulin rows in MT
    !   max events (ev) = 10;   max # event options at position x,y
    !   n of columns (x) = 13;  protofilaments in MT
    !tree size = x * y * ev = 2600000

    !tree**************************************************************
    !tree pointer
    ! extra 2 elements for caping top and bottom of the tree
    integer, dimension(2600002) :: tree_parent, tree_child1, tree_child2
    integer :: treeSize

    !event tree
    !contains information about the event (MT location, event type, time of event)
    integer, dimension(2600002) :: tree_x, tree_y, tree_ev
    double precision, dimension(2600002) :: tree_time, tree_rate
    logical, dimension(2600002) :: tree_never !time time = infinity (or rate = 0)

    !MT matrix that points back to the tree
    integer, dimension(13, 20000, 10) :: mtEv_treeIndx
    integer :: xMax, yMax, evMax, yLimit

    !MT matrix containing information about the MT
    logical, dimension(13, 20000) :: mt_isEmpty, mt_boundToGTP



    !Reaction Rates****************************************************
    !Associaiton
    double precision :: k_assoc
    
    !Dissociation Rates - based upon neighbor context.
    !rate = k_dissoc(x,y)
    !  x = type of longidutinal interaction. 1 = GDP, 2 = GTP
    !  y = [ # non-seam lateral bonds + (# seam lateral bonds*3)]
    !
    !      **Y values 6, 9 are not used. This is because of how y is calculated^^
    !      ** number non-seam bonds is initialized to 1.
    !      ** number of seam lateral initialized to 0.
    !      
    !      'half seam' - subunit is making one seam lateral association.
    !      'full seam' - subunit is making two seam lateral associations.
    !  
    !  Possible Y Values *********************************************************
    !  (no seam lateral bonds):   1 = 0 bonds, 2 = 1 bonds, 3 = 2 bonds
    !  (making a half seam bond): 4 = 0 bonds, 5 = 1 bonds, 6 = 2 bonds (not possible)
    !  (making a full seam bond): 7 = 0 bonds, 8 = 1 bonds, 9 = 2 bonds (not possible)
    !
    !  Y Values translated into configurations **********************************************
    !  (no seam lateral bonds):   1 = loner, 2 = corner, 3 = bucket
    !  (making a half seam bond): 4 = half-seam lateral, 5 = half seam lateral + lateral 
    !  (making a full seam bond): 7 = full seam lateral, 8 = full seam lateral + lateral
    !
    !  Final Examples:
    !  rate = k_dissoc(2,3) = (GTP long, [1 + 2 lateral bonds]) 
    !  rate = k_dissoc(1,3) = (GDP long, [1 + 2 lateral bonds]) 
    !  rate = k_dissoc(2,5) = (GTP long, [1 + 1 lateral bonds + (1 half seam * 3)]) 
    !  rate = k_dissoc(2,7) = (GTP long, [1 + 0 lateral bonds + (2 half seam * 3)]) 
    !
    !  More documentation on line 567
    !
    double precision, dimension(3, 12) :: k_dissoc
    
    !Hydrolysis (GTPase)
    double precision :: k_randHyd
    
    !Averaging window
    integer :: k_conflictRange


    !Input Declare*******************************************************
    double precision, dimension(100) :: cmdLnInput, startInput
    character(len=32) :: cmdLnArg



    !Stats ************************************************************
    integer :: stat_eventN, stat_nTub, stat_nGTP, stat_nCorner, stat_nTrueCorner, stat_nTermGDP
    integer, dimension(13) :: stat_height_GDP, stat_height_tub, stat_GTP_cap
    logical, dimension(13) :: stat_isCorner, stat_isTrueCorner
    
    !Calculating protofilament height statistics
    double precision :: avgHeight, stdHeight



    !Others ***********************************************************
    !current time
    double precision :: time

    !simulation model
    integer :: model
    integer :: startCondition

    !updateList
    integer, dimension(1000) :: upList_x, upList_y, upList_ev
    integer :: nUpdate

    !counters and temporary variables
    integer :: counter1, counter2, counter3, counter4, counter5
    double precision :: temp1, temp2, temp3, temp4, temp5
    double precision, dimension(10) :: temp
    double precision :: temp_rate, temp_time
    integer :: temp_x, temp_y, temp_ev, temp_indx
    double precision :: temp0_rate, temp0_time
    integer :: temp0_x, temp0_y, temp0_ev, temp0_indx
    integer :: parent, child1, child2
    logical :: temp_never, temp0_never
    integer :: temp_nTub
    double precision :: temp_max, temp_min
    integer :: temp_maxHeight, temp_curHeight
    integer, dimension(13) :: temp_tubState

    !simulation limit - by time or events
    double precision :: maxTime
    integer :: eventMax
    logical :: stopIfShort

    !Write frequency
    integer :: writeCond_event, writeCond_size
    double precision :: writeCond_time
    
    logical :: displayAll
    double precision :: dispAll_timeMark
    
    integer :: w_event, w_sizeMin, w_sizeMax
    double precision :: w_time

    !Random seed
    integer :: nseed, clock
    integer, allocatable :: seed(:)
    
    !Reset Counter
    integer :: resetCount



    !Input*******************************************************************************************************
    !See top of the program for a full example

    call getarg(1, cmdLnArg)
    read(cmdLnArg, *) counter2
    counter2 = counter2+1
    
    do counter1 = 2, counter2
        call getarg(counter1, cmdLnArg)
        read(cmdLnArg, *) startInput(counter1-1)
    end do
    do counter1 = (counter2+1), iargc()
        call getarg(counter1, cmdLnArg)
        read(cmdLnArg, *) cmdLnInput(counter1 - counter2)
    end do
    
    !Toggle displayAll on/off
    if (startInput(1) == 1) then
        displayAll = .true.
    else
        displayAll = .false.
    end if
    
    !Start Condition 1 - MT Seed
    startCondition = startInput(2)

    eventMax = startInput(3)
    
    maxTime = startInput(4)
    
    writeCond_event = startInput(5)
    writeCond_time = startInput(6)
    writeCond_size = startInput(7)
    
    if (startInput(8) == 1) then
        stopIfShort = .true.
    else
        stopIfShort = .false.
    end if


    !General Initialization************************************************************************************
    !maxTime = simulation time (in sec) will not exceed maxTime
    !eventMax = simulation will end if number of events reach eventMax

    !Random seed
    nseed = 2
    allocate(seed(nseed))
    call system_clock(count=clock)
    seed = clock + 37 * (/ (counter1-1, counter1=1, nseed) /)
    call random_seed(PUT = seed)

    time = 0

    !Tree and MT Initialization***
    xMax = 13
    yMax = 20000
    evMax = 10
    treeSize = xMax * yMax * evMax
    tree_parent(1) = treeSize + 1

    yLimit = yMax - 50

    counter2 = treeSize + 2

    !tree construct
    !this tree is just a pointer with information on where parent and children are located in the array
    do counter1 = 1, treeSize
        tree_child1(counter1) = min(counter1 * 2, counter2)
        tree_child2(counter1) = min(tree_child1(counter1) + 1, counter2)

        tree_parent(tree_child1(counter1)) = counter1
        tree_parent(tree_child2(counter1)) = counter1
    end do

    !Event tree construct
    !Event tree contains information about where (x,y) event is, what event it is, when the event is, etc.
    !Organized so that time of the parent always smaller than that of the children
    !Also mtEv_treeIndx points from matrix representing MT (with third dimension representing event associated x,y)
    !To corresponding event in the tree
    counter1 = 1
    do temp_ev = 1, evMax
        do temp_y = 1, yMax
            do temp_x = 1, xMax
                tree_time(counter1) = 0
                tree_rate(counter1) = 0
                tree_never(counter1) = .true.
                tree_x(counter1) = temp_x
                tree_y(counter1) = temp_y
                tree_ev(counter1) = temp_ev

                mtEv_treeIndx(temp_x,temp_y,temp_ev) = counter1

                counter1 = counter1 + 1
            end do
        end do
    end do
    
    !top cap of tree
    tree_time(counter1) = 0
    tree_never(counter1) = .false.
    
    !bottom cap of tree
    counter1 = counter1 + 1
    tree_time(counter1) = 0
    tree_never(counter1) = .true.

    !Initialize the microtubule model *************************
    call model1_initiate()

    !write header for fortran output file
    write(6,'(4A12)') 'time    ', 'n_tub    ', 'n_GTP    ', 'temp_max    '


    !Next Step Method**********************************************************************************************
    !set write condition
    w_event = 0
    w_sizeMin = max(18, stat_nTub - writeCond_size)
    w_sizeMax = stat_nTub + writeCond_size
    w_time = writeCond_time
    
    dispAll_timeMark = 100
    
    resetCount = 0
    
    !End simulation if stat_eventN excede eventMax
    do stat_eventN = 1, eventMax

        !set the time = fastest event
        time = tree_time(1)

        !store event
        temp_ev = tree_ev(1)
        temp_x = tree_x(1)
        temp_y = tree_y(1)
        
        ! Write out everything module (if displayAll specified)
        if (displayAll) then
            write(55,'(1F10.5, 14I)') time, stat_nTub, stat_height_tub
        end if

        ! Code to output end structure (if displayAll specified)
        ! NOTE - outputs a left-handed coordinate system!
        if (displayAll .and. time > dispAll_timeMark) then
            dispAll_timeMark = dispAll_timeMark+10
            temp_maxHeight = stat_height_tub(1)
            
            do counter1 = 2, 13
            temp_maxHeight = max(temp_maxHeight, stat_height_tub(counter1))
            end do

            write(50, '(1F10.5, 1I)') time, temp_maxHeight
            
            !Prints MT end structure as follows:
            !Empty = 0, GTP-tubulin = 1, GDP-tubulin = 2
            do counter2 = 0, 30
                temp_curHeight = temp_maxHeight - counter2
                do counter1 = 1, 13
                    if (mt_isEmpty(counter1, temp_curHeight)) then
                        temp_tubState(counter1) = 0
                    else if (mt_boundToGTP(counter1, temp_curHeight)) then
                        temp_tubState(counter1) = 1
                    else
                        temp_tubState(counter1) = 2
                    end if
                end do
                write(50, '(13I)') temp_tubState
            end do
        end if
        

        !Update microtubule and tree according to the model rules
        call model1_rules()

        if (w_event == 0 .or. stat_nTub < w_sizeMin .or. stat_nTub > w_sizeMax .or. time >= w_time) then
            
            !reset write condition
            w_event = writeCond_event
            w_sizeMin = max(18, stat_nTub - writeCond_size)
            w_sizeMax = stat_nTub + writeCond_size
            w_time = time + writeCond_time
            
            !get mean and standard deviation of protofilament heights
            !units in tubulins. tubulin = one alpha-beta heterodimer
            stdHeight = 0
            avgHeight = 0

            !calculate mean protofilament height
            do counter1 = 1,13
                avgHeight = avgHeight + stat_height_tub(counter1)
            end do
            avgHeight = avgHeight / 13

            !calculate standard deviation of protofilament heights
            do counter1 = 1,13
                stdHeight = (dble(stat_height_tub(counter1)) - avgHeight)**2
            end do
            stdHeight = sqrt(stdHeight / 13)

            ! Calculate taper 
            temp_nTub = 0
            do counter1 = 1, 13
                stat_GTP_cap(counter1) = stat_height_tub(counter1) - stat_height_GDP(counter1)
                temp_nTub = max(temp_nTub, stat_height_tub(counter1))
            end do
            temp_max = dble(stat_height_tub(1)) + 0.11538461538
            temp_min = dble(stat_height_tub(1)) + 0.11538461538
            do counter1 = 2, 13
                if (temp_max < dble(stat_height_tub(counter1)) + dble(counter1)*0.11538461538) then
                    temp_max = dble(stat_height_tub(counter1)) + dble(counter1)*0.11538461538
                end if
                if (temp_min > dble(stat_height_tub(counter1)) + dble(counter1)*0.11538461538) then
                    temp_min = dble(stat_height_tub(counter1)) + dble(counter1)*0.11538461538
                end if
            end do
            temp_max = temp_max - temp_min


            ! Write output *******************************************************
            write(6,'(1F10.5, 2I, 1F10.5, 3I)') time, stat_nTub, stat_nGTP, temp_max, stat_nTermGDP, stat_nCorner, stat_nTrueCorner
            


            !Check model reset 
            if (stat_nTub <= 17 .and. resetCount == 0) then
                resetCount = 5
                call model1_reset()
            end if
            if (resetCount > 0) then
                resetCount = resetCount - 1
            end if
            

            !Set other simulation end conditions; print error message
            if (stat_nTub < 18 .and. stopIfShort) then
                write(6, *) '// simulation too short'
                stop
            end if
            if (time > maxTime) then
                write(6, *) '// simulation time limit reached'
                stop
            end if
            do counter1 = 1, 13
                if (stat_height_tub(counter1) > yLimit) then
                    write(6, *) '// simulation height limit reached'
                    stop
                end if
            end do
        end if
        
        !update writeCond_event
        w_event = w_event - 1
    end do
    write(6, *) '// simulation event limit reached'
    stop




    !subroutines and functions******************************************************************************
    contains
    
    !Tree Sorting subroutine ******************************************
    !sorts the tree so that the time of a parent is always smaller than 
    !time of corresponding children
    subroutine sortTree(indx)
        integer :: indx
        parent = tree_parent(indx)
        child1 = tree_child1(indx)
        child2 = tree_child2(indx)

        !go up the tree if applicable
        do while (.not. tree_never(indx) .and. (tree_never(parent) .or. tree_time(indx) < tree_time(parent)))
            call switchNode(indx, parent)
            indx = parent
            parent = tree_parent(indx)
        end do

        !go down the tree if applicable
        do while (.not. tree_never(child1) .and. (tree_never(indx) .or. tree_time(child1)<tree_time(indx)) .or. &
            .not. tree_never(child2) .and. (tree_never(indx) .or. tree_time(child2)<tree_time(indx)))
            if (.not. tree_never(child1) .and. (tree_time(child1) < tree_time(child2) .or. tree_never(child2))) then
                call switchNode(indx, child1)
                indx = child1
                child1 = tree_child1(indx)
                child2 = tree_child2(indx)
            else
                call switchNode(indx, child2)
                indx = child2
                child1 = tree_child1(indx)
                child2 = tree_child2(indx)
            end if
        end do

    end subroutine

    !Used in sortTree subroutine, switches two nodes in a tree. 
    !Also automatically switches/updates mtEv_treeIndx to match the new tree
    !remember, mtEv_treeIndx points to tree corressponding to identical event 
    !in the tree
    subroutine switchNode(indx1, indx2)
        integer :: indx1, indx2

        temp0_x = tree_x(indx1)
        temp0_y = tree_y(indx1)
        temp0_ev = tree_ev(indx1)
        temp0_time = tree_time(indx1)
        temp0_rate = tree_rate(indx1)
        temp0_never = tree_never(indx1)

        tree_x(indx1) = tree_x(indx2)
        tree_y(indx1) = tree_y(indx2)
        tree_ev(indx1) = tree_ev(indx2)
        tree_time(indx1) = tree_time(indx2)
        tree_rate(indx1) = tree_rate(indx2)
        tree_never(indx1) = tree_never(indx2)

        tree_x(indx2) = temp0_x
        tree_y(indx2) = temp0_y
        tree_ev(indx2) = temp0_ev
        tree_time(indx2) = temp0_time
        tree_rate(indx2) = temp0_rate
        tree_never(indx2) = temp0_never

        mtEv_treeIndx(tree_x(indx1), tree_y(indx1), tree_ev(indx1)) = indx1
        mtEv_treeIndx(tree_x(indx2), tree_y(indx2), tree_ev(indx2)) = indx2

    end subroutine

    !Time Generating subroutine ***************************************
    !as title says, generates new time based on new reaction rate for that event
    !assumes newRate is not zero
    !Called by updateTime subroutine which checks that newRate is not 0
    double precision function getNewTime(newRate, oldRate, oldTime)
        double precision :: newRate, oldRate, oldTime, temp1, newTime

        if (oldRate == 0 .or. time <= oldTime) then
            call random_number(temp1)
            newTime = time - log(temp1) / newRate
        else
            if (oldRate == newRate) then
                newTime = oldTime
            else
                newTime = (oldTime - time) * oldRate / newRate + time
            end if
        end if
        getNewTime = newTime
        return
    end function getNewTime

    !UpdateList Subroutines *******************************************
    subroutine updateTime(newRate, x, y, ev)
        integer :: x, y, ev, indx
        double precision :: newRate

        indx = mtEv_treeIndx(x,y,ev)
        if (newRate == 0) then
            tree_time(indx) = 0
            tree_never(indx) = .true.
            tree_rate(indx) = 0
        else
            tree_time(indx) = getNewTime(newRate, tree_rate(indx), tree_time(indx))
            tree_rate(indx) = newRate
            tree_never(indx) = .false.
        end if

        call sortTree(indx)
    end subroutine




    !Model 44 **************************************************************************************************
    
    !Reset ****
    !subroutine to 'reset' the MT matrix
    !repopulates occupied positions in rows 4 and 5 of MT seed with GTP-tubulin
    subroutine model1_reset()
        do temp_x = 1, xMax
            if (.not. mt_isEmpty(temp_x, 4) .and. .not. mt_boundToGTP(temp_x, 4)) then
                stat_nGTP = stat_nGTP + 1
                mt_boundToGTP(temp_x, 4) = .true.
                call model1_updateRate(temp_x, 5, 2)
                call model1_updateRate(temp_x, 4, 3)
            end if
            if (.not. mt_isEmpty(temp_x, 5) .and. .not. mt_boundToGTP(temp_x, 5)) then
                stat_nGTP = stat_nGTP + 1
                mt_boundToGTP(temp_x, 5) = .true.
                call model1_updateRate(temp_x, 6, 2)
                call model1_updateRate(temp_x, 5, 3)
            end if
        enddo
    end subroutine

    !Precalculate rates ********
    !Simply precalculates the event rates for later use, and updates event 
    !times to match initialized MT
    subroutine model1_initiate()
        
        !kon = cmdLnInput(1)                    ! per uM, per second
        !concent = cmdLnInput(2)                ! Molar
        !G_long_GTP = cmdLnInput(3)             ! k_B*T
        !G_long_GDP = cmdLnInput(4)             ! k_B*T
        !G_lat = cmdLnInput(5)                  ! k_B*T
        !G_lat_Seam = cmdLnInput(6)             ! k_B*T
        !G_Bucket = cmdLnInput(7)               ! k_B*T
        !G_Bucket_Seam = cmdLnInput(8)          ! k_B*T    
        !randHyd = cmdLnInput(9) !per sec       ! per second  
	    
        k_assoc = cmdLnInput(1)
        
        ! Convert k_B*T values for use in dissociation rate calculations
        ! Reference: Van Buren et al. 2002, PNAS : our longitudinal value implicitly includes 
        ! the free energy of immobilizing a dimer in the microtubule lattice. 
        temp(1) = exp(cmdLnInput(5))        !non_seam lateral interaction
        temp(2) = sqrt(exp(cmdLnInput(6)))  !seam lateral interaction
        temp(3) = exp(cmdLnInput(7))        !bucket cost at non_seam
        temp(4) = sqrt(exp(cmdLnInput(8)))  !bucket cost at seam
                
        !GDP Longitudinal Interaction  
        !0 half lateral interaction at the seam (or not at seam)
        k_dissoc(1,1) = k_assoc * exp(cmdLnInput(4))
        k_dissoc(1,2) = k_dissoc(1,1) * temp(1)
        k_dissoc(1,3) = k_dissoc(1,2) * temp(1) * temp(3)

        !1 half lateral interaction at the seam
        k_dissoc(1,4) = k_dissoc(1,1) * temp(2)
        k_dissoc(1,5) = k_dissoc(1,2) * temp(2) * temp(4)
        !(1,6) is not possible (2 non_seam interaction + half seam interaction)

        !2 half lateral interactions at the seam
        k_dissoc(1,7) = k_dissoc(1,4) * temp(2)
        k_dissoc(1,8) = k_dissoc(1,5) * temp(2) * temp(4)
        !(1,9) is not possible (2 non_seam interaction + full seam interaction)
        
        !***** GTP Longitudinal Interaction 
        !0 half lateral interaction at the seam (or not at seam)
        k_dissoc(2,1) = k_assoc * exp(cmdLnInput(3))
        k_dissoc(2,2) = k_dissoc(2,1) * temp(1)
        k_dissoc(2,3) = k_dissoc(2,2) * temp(1) * temp(3)

        !1 half lateral interaction at the seam
        k_dissoc(2,4) = k_dissoc(2,1) * temp(2)
        k_dissoc(2,5) = k_dissoc(2,2) * temp(2) * temp(4)

        !2 half lateral interactions at the seam
        k_dissoc(2,7) = k_dissoc(2,4) * temp(2)
        k_dissoc(2,8) = k_dissoc(2,5) * temp(2) * temp(4)

        !Hydrolysis (GTPase) rate 
        k_randHyd = cmdLnInput(9)

        !Calculate on rate
        k_assoc = k_assoc * cmdLnInput(2)
		
        !Averaging window
        k_conflictRange = 0



        !construct MT and set rates and times**************************************
        select case(startCondition)
        	case(1)
                !Create GTP-tubulin seed rows 1 through 5 of MT
                !Creates a blunt MT seed
                do temp_y = 1, 5
                    do temp_x = 1, xMax
                        mt_isEmpty(temp_x, temp_y) = .false.
                        mt_boundToGTP(temp_x, temp_y) = .true.
                	  end do
                end do
                !Initialize emppty rows 6 through yMax of MT
                do temp_y = 6, yMax
                    do temp_x = 1, xMax
                        mt_isEmpty(temp_x, temp_y) = .true.
                        mt_boundToGTP(temp_x, temp_y) = .false.
                    end do
                end do

                !set rate: hydrolysis (event #3)
                !note - rows 1 through 3 do not have a hydrolysis rate.
                !thus mimics a GMP-CPP-stabilized seed
                do temp_y = 4, 5
                    do temp_x = 1, xMax
                        call model1_updateRate(temp_x, temp_y, 3)
                    enddo
                enddo

                !set rate: dissociation (event #2) and association (event #1)
                do temp_x = 1, xMax
                    call model1_updateRate(temp_x, 5, 2)
                    call model1_updateRate(temp_x, 6, 1)
                enddo

                !Initialize microtubule statistics
                stat_nTub = 26
                stat_nGTP = 26

                do counter1 = 1, 1
                    stat_height_GDP = 3
                    stat_height_tub = 5
                end do

                stat_nCorner = 1
                stat_nTrueCorner = 1
                stat_isCorner = .false.
                stat_isCorner(1) = .true.
                stat_isTrueCorner = .false.
                stat_isTrueCorner(1) = .true.
                stat_nTermGDP = 0

            case default
        end select
    end subroutine

    !Subroutine: model 44 event types and rules ******************************************************
    !
    !Updates MT according to given events and determines which event rates may have changed
    !calls model44_updateRate to check that event and update that event time if need be
    !New event types are added here; then require a corresponding new updateRate call

    subroutine model1_rules()
        integer :: counter, y, x, counter2
        temp_x = tree_x(1)
        temp_y = tree_y(1)
        temp_ev = tree_ev(1)
    
        ! three possible events. 
        ! case(1) = association
        ! case(2) = dissociation
        ! case(3) = hydrolysis (GTPase activity)
        
        
        ! When executing event, microtubule stats are first updated. Then, update the 
        ! microtubule lattice then check events whose rates might be affected by the change
        ! update rates if necessary.

        select case(temp_ev)

            !Association
            case(1) 
                
                !update statistics of microtubule
                stat_nTub = stat_nTub + 1
                stat_nGTP = stat_nGTP + 1
                stat_height_tub(temp_x) = temp_y !and height

                !update MT matrix at x,y
                mt_isEmpty(temp_x, temp_y) = .false.
                mt_boundToGTP(temp_x, temp_y) = .true.
                
                !update rates of the tubulin at x,y
                call model1_updateRate(temp_x, temp_y, 1)   !(1) association
                call model1_updateRate(temp_x, temp_y, 2)   !(2) dissociation
                call model1_updateRate(temp_x, temp_y, 3)   !(3) hydrolysis

                !update rates for neighboring tubulin
                call model1_updateRate(temp_x, temp_y+1, 1)
                call model1_updateRate(temp_x, temp_y-1, 2)
                call model1_updateRate(temp_x, temp_y-1, 3)

                !incase the tubulin is at the seam
                do x = temp_x+1,max(temp_x + k_conflictRange,temp_x+1)
                    if (x>13) then
                        call model1_updateRate(x-13, temp_y+0, 2)
                        call model1_updateRate(x-13, temp_y+1, 2)
                        call model1_updateRate(x-13, temp_y+2, 2)
                    else
                        call model1_updateRate(x, temp_y, 2)
                        call model1_updateRate(x, temp_y-1, 2)
                    end if
		 		end do
		 		do x = min(temp_x - k_conflictRange,temp_x-1),temp_x-1
                    if (x<1) then
                        call model1_updateRate(x+13, temp_y-1, 2)
                        call model1_updateRate(x+13, temp_y-2, 2)
                        call model1_updateRate(x+13, temp_y-3, 2)
                    else
                        call model1_updateRate(x, temp_y, 2)
                        call model1_updateRate(x, temp_y-1, 2)
                    end if
		 		end do

            !Dissociation
            case(2)

                !update stats
                stat_nTub = stat_nTub - 1
                stat_height_tub(temp_x) = temp_y - 1

                !Determine if dissociating subunit is GTP-tub or GDP-tub
                if (mt_boundToGTP(temp_x, temp_y)) then
                    stat_nGTP = stat_nGTP - 1
                else
                    counter = temp_y-1
                    do while (mt_boundToGTP(temp_x, counter) .and. (counter>=3))
                        counter = counter - 1
                    end do
                    stat_height_GDP(temp_x) = counter
                end if

                !Update MT at x,y
                mt_isEmpty(temp_x, temp_y) = .true.

                !Update rate calls.
                call model1_updateRate(temp_x, temp_y, 1)
                call model1_updateRate(temp_x, temp_y, 2)
                call model1_updateRate(temp_x, temp_y, 3)
                call model1_updateRate(temp_x, temp_y+1, 1)
                call model1_updateRate(temp_x, temp_y-1, 2)
                call model1_updateRate(temp_x, temp_y-1, 3)

                !Handling seam cases.
		 		do x = temp_x+1,max(temp_x + k_conflictRange,temp_x+1)
                    if (x>13) then
                        call model1_updateRate(x-13, temp_y+0, 2)
                        call model1_updateRate(x-13, temp_y+1, 2)
                        call model1_updateRate(x-13, temp_y+2, 2)
                    else
                        call model1_updateRate(x, temp_y, 2)
                        call model1_updateRate(x, temp_y-1, 2)
                    end if
		 		end do
		 		do x = min(temp_x - k_conflictRange,temp_x-1),temp_x-1
                    if (x<1) then
                        call model1_updateRate(x+13, temp_y-1, 2)
                        call model1_updateRate(x+13, temp_y-2, 2)
                        call model1_updateRate(x+13, temp_y-3, 2)
                    else
                        call model1_updateRate(x, temp_y, 2)
                        call model1_updateRate(x, temp_y-1, 2)
                    end if
		 		end do

            ! Hydrolysis (GTPase)
            case(3)

                !update microtubule stats
                stat_height_GDP(temp_x) = max(temp_y, stat_height_GDP(temp_x))
                stat_nGTP = stat_nGTP - 1

                !update microtubule at x,y
                mt_boundToGTP(temp_x, temp_y) = .false.

                !update rates
                call model1_updateRate(temp_x, temp_y, 3)
                call model1_updateRate(temp_x, temp_y+1, 2)

                !handling seam cases
		 		do x = temp_x+1,temp_x + k_conflictRange
                    if (x>13) then
                        call model1_updateRate(x-13, temp_y+2, 2)
                        call model1_updateRate(x-13, temp_y+3, 2)
                    else
                        call model1_updateRate(x, temp_y+1, 2)
                    end if
		 		end do
		 		do x = temp_x - k_conflictRange,temp_x-1
                    if (x<1) then
                        call model1_updateRate(x+13, temp_y-1, 2)
                        call model1_updateRate(x+13, temp_y, 2)
                    else
                        call model1_updateRate(x, temp_y+1, 2)
                    end if
		 		end do

            case default
        end select

        !determine number of GDP tubulins on microtubule end
        !number terminal GDPs; column 5 in output
        stat_nTermGDP = 0
        do counter = 1,13
            if (.not. mt_boundToGTP(counter,stat_height_tub(counter))) then
                stat_nTermGDP = stat_nTermGDP + 1
            end if
        end do
        
        !determine number and type of corner sites on microtubule end
        !corner vs. true corner distinguished by neighbor context
        !written in simulation output columns 6,7
        x = temp_x
        do counter = 1,3
            x = temp_x + counter - 2
            y = stat_height_tub(x)+1

            !Handles seam at x=1, and also non-seam at x = 2
            if (x == 1 .or. x == 14) then
                x=1
                if (.not. mt_isEmpty(13,y-1) .or. .not. mt_isEmpty(2,y)) then
                    stat_isCorner(x) = .true.
                    if (.not. mt_isEmpty(13,y-1) .and. .not. mt_isEmpty(12,y-1) .or. &
                    .not. mt_isEmpty(2,y) .and. .not. mt_isEmpty(3,y)) then
                        stat_isTrueCorner(x) = .true.
                    else
                        stat_isTrueCorner(x) = .false.
                    end if
                else
                    stat_isCorner(x) = .false.
                    stat_isTrueCorner(x) = .false.
                end if
            !Handles seam at x=13, and also non-seam at x=12
            else if (x == 13 .or. x == 0) then
                x=13
                if (.not. mt_isEmpty(12,y) .or. .not. mt_isEmpty(1,y+2)) then
                    stat_isCorner(x) = .true.
                    if (.not. mt_isEmpty(12,y) .and. .not. mt_isEmpty(11,y) .or. &
                    .not. mt_isEmpty(1,y+2) .and. .not. mt_isEmpty(2,y+2)) then
                        stat_isTrueCorner(x) = .true.
                    else
                        stat_isTrueCorner(x) = .false.
                    end if
                else
                    stat_isCorner(x) = .false.
                    stat_isTrueCorner(x) = .false.
                end if
            !Non-seam cases
            else
                if (.not. mt_isEmpty(x-1,y) .or. .not. mt_isEmpty(x+1,y)) then
                    stat_isCorner(x) = .true.
                    if (.not. mt_isEmpty(x-1,y) .and. .not. mt_isEmpty(x-2,y) .or. &
                    .not. mt_isEmpty(x+1,y) .and. .not. mt_isEmpty(x+2,y)) then
                        stat_isTrueCorner(x) = .true.
                    else
                        stat_isTrueCorner(x) = .false.
                    end if
                else
                    stat_isCorner(x) = .false.
                    stat_isTrueCorner(x) = .false.
                end if
            end if
        end do

        counter = 0
        counter2 = 0
        
        do x = 1, 13
            if (stat_isCorner(x)) then
                counter = counter+1
            end if
            if (stat_isTrueCorner(x)) then
                counter2 = counter2+1
            end if
        end do
        stat_nCorner = counter
        stat_nTrueCorner = counter2

    end subroutine

    !Calculate rates***********************************************
    !determines what new rate is and changes event time accordingly
    subroutine model1_updateRate(x, y, ev)
        integer :: x, y, ev, latBond, seamBond
        double precision :: rate
        integer :: temp_x

        ! cases correspond to the three possible events.
        ! case(1) = association
        ! case(2) = dissociation
        ! case(3) = hydrolysis (GTPase)

        !determine the rate
        select case(ev)

            ! Association
            case(1)
                if (.not. mt_isEmpty(x,y) .or. mt_isEmpty(x,y-1)) then
                    rate = 0
                else
                    rate = k_assoc
                end if

            ! Dissociation
            case(2)
                
                if (y <= 3 .or. mt_isEmpty(x,y) .or. .not. mt_isEmpty(x,y+1)) then
                    rate = 0
                else
                    !Characterize interactions of dissociating tubulin at x,y
                    latBond = 1
                    seamBond = 0
                    
                    !Handling seam cases
                    if (x==1) then
                        if (.not. mt_isEmpty(13,y-1)) then

                            seamBond = 2
                        elseif (.not. mt_isEmpty(13,y-2)) then
                            seamBond = 1
                        else
                        end if
                    !Check non-seam
                    else
                        if (.not. mt_isEmpty(x-1,y)) then
                            latBond = latBond + 1
                        else
                        end if
                    end if
                    !Handling seam cases
                    if (x==13) then
                        if (.not. mt_isEmpty(1,y+2)) then
                            seamBond = 2
                        elseif (.not. mt_isEmpty(1,y+1)) then
                            seamBond = 1
                        else
                        end if
                    !Non-seam
                    else
                        if (.not. mt_isEmpty(x+1,y)) then
                            latBond = latBond + 1
                        else
                        end if
                    end if
                    
                    !calculate rate *************************************
                    !Determine nucleotide occupancy of subunit below x,y
                    !Use longitudinal interaction + lateral interactions 
                    ! (neighbor context) to call appropriate dissociation rate

                    if (mt_boundToGTP(x,y-1)) then
                        rate = k_dissoc(2,latBond + seamBond*3)
                    else
                        rate = k_dissoc(1,latBond + seamBond*3)
                    end if
                end if
            
            ! Hydrolysis (GTPase)
            case(3)
                if (y > 3 .and. .not. mt_isEmpty(x,y) .and. mt_boundToGTP(x,y) .and. .not. mt_isEmpty(x,y+1)) then
                    rate = k_randHyd
                else
                    rate = 0
                end if
            case default
        endselect

        !Update the time for that event.
        call updateTime(rate, x, y, ev)
    end subroutine

end program simulateMT
